#include <stm32f072xb.h>
#include <dma.hpp>
#include <gpio.hpp>
#include <uart-gpio.hpp>
#include <uart.hpp>

using namespace stm32f072xb;

AbstractUartInit uartInit = {WordLength::HEIGHT_BITS, StopBits::ONE_BIT, 115200};
UartGpio<UartRegister::UART2, UartGpioType::RX, Port::A> rxGpio;
UartGpio<UartRegister::UART2, UartGpioType::TX, Port::A> txGpio;

Uart<UartRegister::UART2, Port::A> uartTxRx(uartInit, txGpio, rxGpio);

constexpr uint8_t bufferSize = 5;
uint8_t buffer[bufferSize] = {0};

AbstractDmaInit dmaUartRxInit = {DmaPeripheral::USART2_RX,
                                 DmaDirection::PERIPHERAL_TO_MEMORY,
                                 DmaCircularMode::ENABLE,
                                 DmaMemoryIncrement::ENABLE,
                                 DmaMemorySize::BYTE,
                                 DmaPeripheralIncrement::DISABLE,
                                 DmaPeripheralSize::BYTE,
                                 buffer,
                                 bufferSize};
Dma<DmaChannel::CHANNEL5, DmaPeripheral::USART2_RX> dmaUartRx(dmaUartRxInit, uartTxRx);
AbstractDmaInit dmaUartTxInit = {DmaPeripheral::USART2_TX,
                                 DmaDirection::MEMORY_TO_PERIPHERAL,
                                 DmaCircularMode::DISABLE,
                                 DmaMemoryIncrement::ENABLE,
                                 DmaMemorySize::BYTE,
                                 DmaPeripheralIncrement::DISABLE,
                                 DmaPeripheralSize::BYTE,
                                 buffer,
                                 bufferSize};

Dma<DmaChannel::CHANNEL4, DmaPeripheral::USART2_TX> dmaUartTx(dmaUartTxInit, uartTxRx);

extern "C" void DMA1_Channel4_5_6_7_IRQHandler() {
	if (READ_BIT(DMA1->ISR, DMA_ISR_TEIF6)) {  // DMA Error occured
		SET_BIT(DMA1->IFCR, DMA_IFCR_CTEIF6);  // Clear interrupt register
	}

	if (READ_BIT(DMA1->ISR, DMA_ISR_TCIF5)) {
		SET_BIT(DMA1->IFCR, DMA_IFCR_CTCIF5);
		dmaUartTx.start();
	}
}

void enableDmaChannel4_5_6_7Irq();

int main() {
	SET_BIT(DMA1_Channel5->CCR, DMA_CCR_TCIE);
	enableDmaChannel4_5_6_7Irq();
	dmaUartRx.start();

	while (true) {
	}
}

void enableDmaChannel4_5_6_7Irq() {
	NVIC_EnableIRQ(DMA1_Channel4_5_6_7_IRQn);
	NVIC_SetPriority(DMA1_Channel4_5_6_7_IRQn, 0);
}
